`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/02 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`define ENABLE 1'b1
module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst_n,
    select_mode,
    select_clk,
    select_sec_min
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    input clk;
    input rst_n;
    input select_mode;
    input select_clk;
    input select_sec_min;
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire debounced_in, debounced_in_clk, debounced_in_sec_min;
    wire out_pulse, out_pulse_clk, out_pulse_sec_min;

    wire clk_out_for_count;

    wire enable_min;

    wire [3:0] digit_1_sec, digit_0_sec;    
    wire [3:0] digit_3, digit_2, digit_1, digit_0;
    wire [3:0] hour_digit_0, hour_digit_1;

    wire AM_PM_convert;
    wire curr_state, curr_state_clk, curr_state_sec_min;

    wire [3:0] ssd_in;
    wire [3:0] ssd_in_min, ssd_in_sec;
    wire [3:0] ssd_ctl_min, ssd_ctl_sec;
    wire [7:0] D_ssd_a, D_ssd_b;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst_n(rst_n)   //low-active reset
    );

    debounce U01( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_mode),
    .pb_debounced(debounced_in)
    );

    debounce U011( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_clk),
    .pb_debounced(debounced_in_clk)
    );

    debounce U0111( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_sec_min),
    .pb_debounced(debounced_in_sec_min)
    );

    one_pulse U02( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in),
    .out_pulse(out_pulse)
);
    one_pulse U022( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_clk),
    .out_pulse(out_pulse_clk)
);
    one_pulse U0222( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_sec_min),
    .out_pulse(out_pulse_sec_min)
);

    
    fsm U03(
    .select_mode_change(out_pulse),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state)
    );

    fsm U033(
    .select_mode_change(out_pulse_clk),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_clk)
    );

    fsm U0333(
    .select_mode_change(out_pulse_sec_min),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_sec_min)
    );

    select_clk U03333(
    .clk_1Hz(clk_1Hz),
    .clk_100Hz(clk_100Hz),
    .clk_out_for_count(clk_out_for_count),
    .select(curr_state_clk)
);

    BCD_up_2d U04(
    .digit_1(digit_1_sec),
    .digit_0(digit_0_sec),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .carry_out(enable_min),
    .en(`ENABLE)
    );

    BCD_up_4d U1(
    .digit_3(digit_3),
    .digit_2(digit_2),
    .digit_1(digit_1),
    .digit_0(digit_0),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .AM_PM_convert(AM_PM_convert),
    .en(enable_min)
    );

    AM_PM_selector U2(
    .hour_in_1(digit_3),
    .hour_in_0(digit_2),
    .hour_out_1(hour_digit_1),
    .hour_out_0(hour_digit_0),
    .AM_PM_convert(AM_PM_convert),
    .select_mode(curr_state)
    );
   
    scan_freq_ctl U3(
    .ssd_in(ssd_in_min),
    .ssd_ctl(ssd_ctl_min),
    .in_0(digit_0),
    .in_1(digit_1),
    .in_2(hour_digit_0),
    .in_3(hour_digit_1),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    scan_freq_ctl U33(
    .ssd_in(ssd_in_sec),
    .ssd_ctl(ssd_ctl_sec),
    .in_0(digit_0_sec),
    .in_1(digit_1_sec),
    .in_2(4'd0),
    .in_3(4'd0),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    ssd_selector U333(
    .ssd_ctl(ssd_ctl),
    .ssd_in(ssd_in),
    .ssd_ctl_min(ssd_ctl_min),
    .ssd_ctl_sec(ssd_ctl_sec),
    .ssd_in_min(ssd_in_min),
    .ssd_in_sec(ssd_in_sec),
    .select_mode(curr_state_sec_min)
);
    
    display U4(.D_ssd(D_ssd_a), .i(ssd_in));

    display_with_point U5(.D_ssd(D_ssd_b), .i(ssd_in));

    display_selector U6(
    .D_ssd(D_ssd),
    .D_ssd_a(D_ssd_a),
    .D_ssd_b(D_ssd_b),
    .select_mode(curr_state),
    .AM_PM_convert(AM_PM_convert),
    .clk_ssd_ctl(clk_ssd_ctl)
    );  
endmodule
